﻿using System.Linq;
using System.Windows;
using System.Windows.Media;

using MindFusion;
using MindFusion.Diagramming.Wpf;

using Shape = MindFusion.Diagramming.Wpf.Shape;


namespace LinkDragDrop
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		public MainWindow()
		{
			InitializeComponent();

			var connectorShape = new Shape(
				null, // no borders
				new[] // decorations
				{
					new LineTemplate(10, 10, 10, 50),
					new LineTemplate(10, 50, 90, 50),
					new LineTemplate(90, 50, 90, 90)
				},
				null,
				FillRule.Nonzero, "Connector");

			var item1 = new ShapeNode { Shape = Shapes.Rectangle };
			NodeListView.SetLabel(item1, "Activity");
			nodeList.Items.Add(item1);

			var item2 = new ShapeNode { Shape = Shapes.Decision };
			NodeListView.SetLabel(item2, "Decision");
			nodeList.Items.Add(item2);

			var item3 = new ShapeNode { Shape = connectorShape };
			NodeListView.SetLabel(item3, "Connector");
			nodeList.Items.Add(item3);
		}

		private void OnNodeCreated(object sender, NodeEventArgs e)
		{
			var node = e.Node as ShapeNode;
			if (node != null)
			{
				if (node.Shape.Id == "Connector")
				{
					// replace the dummy connector node with a DiagramLink
					var bounds = node.Bounds;
					diagram.Items.Remove(node);

					var link = diagram.Factory.CreateDiagramLink(
						bounds.TopLeft, bounds.BottomRight);
					link.SegmentCount = 3;
					link.Shape = LinkShape.Cascading;

					ConnectToNearbyNode(link);
				}
				else
				{
					node.AnchorPattern = AnchorPattern.Decision2In2Out;
					node.Effects.Add(new GlassEffect());
					ConnectToNearbyLink(node);
				}
			}
		}

		private void ConnectToNearbyNode(DiagramLink link)
		{
			// connect to a nearby origin node
			var origin = diagram.Nodes.OrderBy(n =>
				Utilities.Distance(n.GetCenter(), link.StartPoint)).FirstOrDefault();

			if (origin != null)
			{
				var distance = Utilities.Distance(origin.GetCenter(), link.StartPoint);
				if (distance < 120)
				{
					link.Origin = origin;
					link.Route();
				}
			}

			// connect to a nearby destination node
			var destination = diagram.Nodes.Where(n => n != origin).OrderBy(n =>
				Utilities.Distance(n.GetCenter(), link.EndPoint)).FirstOrDefault();

			if (destination != null)
			{
				var distance = Utilities.Distance(destination.GetCenter(), link.EndPoint);
				if (distance < 120)
				{
					link.Destination = destination;
					link.Route();
				}
			}
		}

		private void ConnectToNearbyLink(DiagramNode node)
		{
			var outLink = diagram.Links.Where(l => l.Origin is DummyNode).OrderBy(l =>
				Utilities.Distance(node.GetCenter(), l.StartPoint)).FirstOrDefault();

			if (outLink != null)
			{
				var distance = Utilities.Distance(node.GetCenter(), outLink.StartPoint);
				if (distance < 90)
				{
					outLink.Origin = node;
					outLink.Route();
					return;
				}
			}

			var inLink = diagram.Links.Where(l => l.Destination is DummyNode).OrderBy(l =>
				Utilities.Distance(node.GetCenter(), l.EndPoint)).FirstOrDefault();

			if (inLink != null)
			{
				var distance = Utilities.Distance(node.GetCenter(), inLink.EndPoint);
				if (distance < 90)
				{
					inLink.Destination = node;
					inLink.Route();
					return;
				}
			}
		}

		private void OnNodeModified(object sender, NodeEventArgs e)
		{
			ConnectToNearbyLink(e.Node);
		}
	}
}